Enum budvm::Comparison
source · pub enum Comparison {
Equal,
NotEqual,
LessThan,
LessThanOrEqual,
GreaterThan,
GreaterThanOrEqual,
}Expand description
A method for comparing Values.
Variants
Equal
Pushes true if left and right are equal. Otherwise, pushes false.
NotEqual
Pushes true if left and right are not equal. Otherwise, pushes false.
LessThan
Pushes true if left is less than right. Otherwise, pushes false.
LessThanOrEqual
Pushes true if left is less than or equal to right. Otherwise, pushes false.
GreaterThan
Pushes true if left is greater than right. Otherwise, pushes false.
GreaterThanOrEqual
Pushes true if left is greater than or equal to right. Otherwise, pushes false.