Struct budvm::StringLiteralDisplay
source · pub struct StringLiteralDisplay<'a>(_);Expand description
A Display implementor that converts a string value to its literal form
including wrapping double quotes.
pub struct StringLiteralDisplay<'a>(_);A Display implementor that converts a string value to its literal form
including wrapping double quotes.